/*-------------------------------------------------------------------------
    This source belongs to CAD vectorial fonts package Ver 3.01
    Daniel Inc Copyrigth (c) 1993
    
    Please feel free to copy the executable demo file, this source file and 
    the vectorial font files that comes with the demo package without
    modifications.
------------------------------------------------------------------------*/    


/* An example of a CNC generator from vectorial font data */

#include "stdlib.h"
#include "stdio.h"
#include "conio.h"
#include "graph.h"
#include "dos.h"
#include "string.h"
#include "math.h"


#include "..\..\include\fontlib.h"

struct TextStyle Normal={ "Normal",          /* definition of Style */      
                          "cla4.fnt",        /* definition of the font file */
                           0.0,              /* heigth         RESERVED */   
                           1.0,              /* with           RESERVED */   
                           0.0,              /* Obiquing angle RESERVED */   
                           FALSE,            /* BackWard       RESERVED */   
                           FALSE,            /* UpsideDown     RESERVED */   
                           HORIZONTAL};      /* initialized to HORIZONTAL */


void InitTextMode(void); 
int  tg_getmaxy(void); 
void InitGraphics(void); 
void PutDisplay(FILE *Cnc);
int FontFileExists(char *FontFile);
size_t FontFileSize(char *FontFile);
void CallPause(void);
void AnalizeAndChange(char *Str);

#define MAX_NUM_PRIMITIVES_FILE 2000
struct primitivefile Display[MAX_NUM_PRIMITIVES_FILE];

TEXTOUPUTSTATE  OuttextState;  /* structure returned by Text() function */
#define DisplayIndex (OuttextState.LastPrimitiveIndex)

char * BufferFonts;
unsigned long Counter=0;

int Pause=FALSE;

/*--------------------------------------------------------------------------*/
void main(int argc, char *argv[]);
void main(int argc, char *argv[])
{ 
size_t FontFileLength;
FILE *fp;
FILE *Cnc;



if(argc>=2)
  {
  if(argv[1][0]!='-')
    {
    strcpy(Normal.FontFile,argv[1]);
    }
    else
     if(tolower(argv[1][1]=='p'))
      Pause=TRUE;
  }

if(argv[2][0]=='-'&& tolower(argv[2][1])=='p')
  Pause=TRUE;

/* loading default font file */
    
if(FontFileExists(Normal.FontFile))               /* see if its on disk */       
  FontFileLength=FontFileSize(Normal.FontFile);   /* get the size */
else
  {
  cputs("Font file does not exist\r\n");
  exit(0);  
  }
      
/* get some memory for the buffer that will hold the font file */
if((BufferFonts=( char *) calloc(sizeof( char),(size_t)FontFileLength))==NULL)
  {
  cputs("NO Memory in text for bufferfonts \r\r\n");
  exit(0);
  }
     
      
/* Initializing the font file, this function loads the font file,
  builds a internal table with each font, so execution is faster */
      
   
UserSetPF(Display , MAX_NUM_PRIMITIVES_FILE);   
   
if(InitStyle(Normal,BufferFonts,FontFileLength)!=OK)
 {   
 cputs("\r\nImpossible to Open Font File or Not Enough Memory");
 exit(1);
 }               
 
 cputs("\r\nSintax C:>CNC <font file> <-P>\r\n\r\n");
 cputs("If no font file is specified ten cla4.fnt is used\r\n");
 cputs("If -p is specified then the CNC will execute by pressing Return\r\n\r\n");
 cputs("Example: C:>cnc let2.fnt -p\r\n");
 cputs("         C:>cnc -p         \r\n");
 cputs("         C:>cnc            \r\n\r\n");

 cputs("This is a CNC code generator that can be used by a \r\n");
 cputs("milling machine, it simulates the rapid movement of the tool\r\n");
 cputs("and the linear interpolation. The code will be displayed\r\n");
 cputs("and sent to cnccode.cnc file\r\n\r\n");
 cputs("The red lines represent the rapid movement of the tool G00\r\n");
 cputs("The white lines represent the linear interpolation G01\r\n");
 cputs("Please press any key to begin");
 getch();

 InitGraphics(); /* graphic initialization */

 Cnc=fopen("cnccode.cnc","w");
 fprintf(Cnc,"; CNC Code for Milling Machines.\n\n");
 
 
 if((fp=fopen("cnctext.txt","r"))!=NULL)
    {
    char Buffer[255];
    int XPos=10;
    double Heigth=50.0;
    double Angle=0.0;
    RW_POINT Pt1;
    RW_POINT Pt2;

    
    Pt1.x=XPos;
    Pt1.y=300;
    
    while(!feof(fp)) /* while end of file not reached */
         {
         
         if((fgets(Buffer,255,fp))!=NULL)
           {

           /* Here comes the function that changes the text (Extended code)*/
              AnalizeAndChange(Buffer);


           /* Buffer has a \n at the end */
           OuttextState=Text(
             Buffer,               /* string to output */
             DEFAULT,              /* the justification is in the header */
             Pt1,                  /* first point */
             Pt2,                  /* second point */
             Heigth,               /* Heigth of the font */
             Angle*(3.1415/180.0), /* Angle to rotate (Radians)*/
             BufferFonts );        /* The Buffer where the font is loaded */
          
           PutDisplay(Cnc); /* display the string */
           
           /* after the string has been displayed the \n works as a line feed
              at the end makes the Y position point to the next line, but
              the x value must be set again to the initial x value
              */
           
          
           /* Pt1.x does not change, it keeps the initial value */
           Pt1.y=OuttextState.LastPt.y;
           }
         }  
    }
    else
        cputs("Cant open cnctext.txt from disk\r\n");


fprintf(Cnc,"\n; End of the code.\n\n");
fclose(Cnc);        
fclose(fp);
_settextposition(25,5);
cputs("That's all folk's                                            ");
getch();
InitTextMode();
cputs("File cnccode.cnc has all the CNC code, load it with an ASCII editor\n");

}                        
#define WHITE 15
#define RED 4
/*-----------------------------------------------------------------------*/
void PutDisplay(FILE *Cnc);
void PutDisplay(FILE *Cnc)
{
   unsigned long i=0;
   int y=tg_getmaxy();  
   char Buffer[255];
   
   static ToolIsUp=TRUE;
    
   /* Trace from the first element in the table (0) to the last one (defined)
      in the variable LastPrimitiveIndex returned by Text(), LastPrimitiveIndex 
      is updated wen the Text() function is called 
    */  
   for(i=0;i<DisplayIndex;i++)   
      {                                    
            
      ++Counter;  /* CNC line Counter */
       if(Display[i].OpCode==PFMOVETO)    /* Rapid movement of the tool */
         {
          /* CODE GENERATION */                                                    
          
          if(ToolIsUp==FALSE) /* we must raise the tool or the workpiece will be damage */
            {
            /* raise the tool 50mm */
            /* Notice that X and Y are 0 so the tool only moves in the vertical axis */
            sprintf(Buffer,"N%ld  G00 X0 Y0 Z%d   Movement of the tool in Z axis \n",Counter*10,50);
            _settextposition(23,5);
            cputs(Buffer);
            sprintf(Buffer,"N%ld  G00 X0 Y0 Z%d\n",Counter*10,50);
            fprintf(Cnc,"%s",Buffer);
            CallPause();

            Counter++;      
            ToolIsUp=TRUE;
            }
          /* Rapid movement of the tool */
          sprintf(Buffer,"N%ld  G00 X%hd Y%hd     Rapid movement of the tool         \n",Counter*10,(short)Display[i].x,(short)Display[i].y);
          _settextposition(23,5);
          cputs(Buffer);
          sprintf(Buffer,"N%ld  G00 X%hd Y%hd\n",Counter*10,(short)Display[i].x,(short)Display[i].y);
          fprintf(Cnc,"%s",Buffer);



           /* SIMULATION */
             _setcolor(RED);
             /* draw to the position */
            _lineto((short)Display[i].x,y-(short)Display[i].y);  /* y- stands for invertin axes */

          CallPause();
         }

       else if(Display[i].OpCode==PFLINETO)   /* Display[i].OpCode==PFLINETO */
          {
           /* CODE GENERATION */
                      
           if(ToolIsUp==TRUE) /* we must lower the tool so the workpiece will be machined */
            {
            /* lower the tool 50mm (the same length it was raised to make the rapid movement) */
            /* Notice that X and Y are 0 so the tool only moves in the vertical axis */
            sprintf(Buffer,"N%ld  G00 X0 Y0 Z%d   Movement of the tool in Z axis \n",Counter*10,-50);
            _settextposition(23,5);
            cputs(Buffer);
            sprintf(Buffer,"N%ld  G00 X0 Y0 Z%d\n",Counter*10,-50);
            fprintf(Cnc,"%s",Buffer);
            CallPause();

            Counter++;      
            ToolIsUp=FALSE;
            }
          
           /* Linear interpolation of the tool */
           sprintf(Buffer,"N%ld  G01 X%hd Y%hd     Linear Interpolation          \n",Counter*10,(short)Display[i].x,(short)Display[i].y);
           _settextposition(23,5);
           cputs(Buffer);
           sprintf(Buffer,"N%ld  G01 X%hd Y%hd\n",Counter*10,(short)Display[i].x,(short)Display[i].y);
           fprintf(Cnc,"%s",Buffer);


          /* SIMULATION */
           _setcolor(WHITE);
           /* draw to the position */
           _lineto((short)Display[i].x,y-(short)Display[i].y);  /* y- stands for invertin axes */
           CallPause();
           }
        
      }
}


/*-----------------------------------------------------------------------*/

/*-----------------------------------------*/
void CallPause(void);
void CallPause(void)
{     
int Get=0;
if(Pause==TRUE)
  {
  _settextposition(25,5);
  cputs("Pause is ON, press Return to continue");
  Get=getch();
  if(Get==27)
    {
    fcloseall(); /* Get the heck out of here */
    InitTextMode();
    cputs("That key ends my execution......... bye\r\n");
    exit(0);
    }
    
  }
}
/*---------------------------------------------------------------------*/
void InitTextMode(void);  /* go back to text mode */
void InitTextMode(void)
{
  _setvideomode(_DEFAULTMODE);
}
/*---------------------------------------------------------------------*/
/* Returns the max number of pixels in y cood */
int tg_getmaxy(void); 
int tg_getmaxy(void)
{                
struct videoconfig config;

  _getvideoconfig(&config);
   return(config.numypixels);
}

/*---------------------------------------------------------------------*/
void InitGraphics(void); /* begin graphic mode */
void InitGraphics(void)
{                
  _settextcolor(7);
  _settextposition(1,1);
  _clearscreen(_GCLEARSCREEN);

  /*                                                     
  if (_setvideomode(_VRES16COLOR))  Its nicer with an EGA!!
   return;
  */
  if (_setvideomode(_ERESCOLOR))
   return;
  
  cputs("This demo program  only runs in VGA or EGA Display\r\n");
  exit(0);
}
/*---------------------------------------------------------------------*/
/*------------------------------------------------------------------------*/
int FontFileExists(char *FontFile);
int FontFileExists(char *FontFile)
{
struct find_t Buffer;
return(!_dos_findfirst(FontFile,_A_NORMAL,&Buffer));
}

/*-------------------------------------------------------*/
size_t FontFileSize(char *FontFile);
size_t FontFileSize(char *FontFile)
{
struct find_t Buffer;
_dos_findfirst(FontFile,_A_NORMAL,&Buffer);
return((size_t)Buffer.size);
}
/*------------------------------------------------------------------------*/

void AnalizeAndChange(char *Str);
void AnalizeAndChange(char *Str)
{
char Buffer[80];       /* new formatted string buffer */
char Number[5];        /* holds the ascii value */
int  MyNumber=0;       /* holds the ascii number */
int length=strlen(Str);
int j=0,i=0;

for (i=0;i<=length;) /* analize all characters */
    {
    if(Str[i]=='\\')  /* if first ocurrence of '\'found */
      {
      if(Str[i+1]=='\\') /* if the next character is also a '\' then this is an extended code format */
        {
        Number[0]=Str[i+2];  /* take the next three numbers */
        Number[1]=Str[i+3];
        Number[2]=Str[i+4];
        Number[3]='\0';

        MyNumber=atoi(Number);      /* convert it to an int */
        Buffer[j]=(char)MyNumber;   /* cast that int to a char type */
        i+=5,j++;;                  /* jumps he special code: \\XXX */
      }
      else /* only one \ found, so copy it just as it is */
        {
        Buffer[j]=(char)Str[i];
        i++,j++;
        }
      }
      else  /* Normal Ascii code found */
        {
        Buffer[j]=(char)Str[i];
        i++,j++;
        }
      }

Buffer[j]='\0';      /* end the string with a nice NULL */
strcpy(Str,Buffer);  /* modify the string passed to the function */
}
